#
# 3つの Annual Cycle のみを維持したデータを作成
# 365days, 182.5days, 122days
#
#     Created by Eriko Nishimoto
#

require 'numru/gphys'
include NumRu

#---------------------- original data open
dir = './'
fname = 'olr.day.mean.nc'
var="olr"

st = Date.parse('1979-01-01')
et = Date.parse('2011-12-31')

data = GPhys::IO.open(dir+fname,var)\
  .cut('time'=>st..et,'lat'=>-30..30)

#---------------------- setting output file
outdir="./"
outfname="AnnualCycle.nc"
outfile = NetCDF.create("#{outdir}#{outfname}")
outfname2="daily_rmAC.nc"
outfile2 = NetCDF.create("#{outdir}#{outfname2}")

#---------------------- 対象とする周期
#cycle = NArray[365.25,182.625,121.75]
# nt/freq = cycle <=> freq=nt/cycle
freq = NArray[33.0,66.0,99.0].to_i
cycle = nt/freq.to_f
p cycle

#-------------------------- calculation & output
### forward FFT with third dim and normalization
GPhys::fft_ignore_missing(true)
fc = data.fft(false,-1)

nx,ny,nt = fc.shape
typecode = fc.typecode

i=0
freq.each{|ff|
  #-------------------- 特定の周期のみを保持したデータを計算
  dum = NArray.new(typecode,nx,ny,nt).fill(0.0)
  dum[false,ff] = fc[false,ff].val
  dum[false,-ff] = fc[false,-ff].val
  bp_data = fc.copy.replace_val(dum)

  ### backward FFT with third dim
  gphys = bp_data.fft(true,-1).real

  #-------------------- 特定の周期のみを保持したデータを出力
  ### make NetCDF
  longname = "olr_with_#{format("%.02f",cycle[i])}days"
  gphys.name = "olr#{i}"
  gphys.long_name = longname
  # 要らないattribution名を除去
  gphys.del_att('missing_value')
  gphys.del_att('actual_range')
  gphys.del_att('valid_range')
  gphys.del_att('parent_stat')
  gphys.del_att('level_desc')
  gphys.del_att('precision')
  gphys.del_att('statistic')
  GPhys::NetCDF_IO.write(outfile,gphys)

  #-------------------- 特定の周期を除去
  data = data - gphys

  i+=1
}
outfile.close

#-------------------- 特定の周期を除去したデータを出力
data.del_att('missing_value')
data.del_att('actual_range')
data.del_att('valid_range')
GPhys::NetCDF_IO.write(outfile2,data)
data_detrend = data.detrend(-1)
data_detrend.rename('olr_detrend')
data_detrend.long_name='olr_detrend'
GPhys::NetCDF_IO.write(outfile2,data_detrend)
outfile2.close


